#!/usr/bin/env python
# Copyright 2015-2018 Yelp Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import argparse
import os
import sys

from paasta_tools.envoy_tools import are_namespaces_up_in_eds
from paasta_tools.envoy_tools import are_services_up_in_pod as is_envoy_ready
from paasta_tools.smartstack_tools import (
    are_services_up_on_ip_port as is_smartstack_ready,
)
from paasta_tools.utils import load_system_paasta_config


system_paasta_config = load_system_paasta_config()

synapse_port = system_paasta_config.get_synapse_port()
synapse_host = "169.254.255.254"
synapse_haproxy_url_format = system_paasta_config.get_synapse_haproxy_url_format()

envoy_host = os.environ["PAASTA_HOST"]
envoy_admin_port = system_paasta_config.get_envoy_admin_port()
envoy_admin_endpoint_format = system_paasta_config.get_envoy_admin_endpoint_format()
envoy_eds_path = "/nail/etc/envoy/endpoints"
pod_ip = os.environ["PAASTA_POD_IP"]

###############################################################
#
# This file is used in the hacheck sidecar, make sure to update `check_proxy_up.sh`
# when changing this file
#
###############################################################


def get_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser()

    parser.add_argument(
        "--enable-smartstack",
        dest="smartstack_readiness_check_enabled",
        action="store_true",
        help="Check smartstack readiness",
    )

    parser.add_argument(
        "--enable-envoy",
        action="store_true",
        dest="envoy_readiness_check_enabled",
        help="Check envoy readiness",
    )

    parser.add_argument(
        "--envoy-check-mode",
        choices=["admin-port", "eds-dir"],
        default="admin-port",
        help="Query Envoy backends through the admin interface (default) or the EDS directory",
    )

    parser.add_argument(
        "pod_port",
        help="Pod Port",
        type=int,
    )

    parser.add_argument(
        "services",
        nargs="+",
        help="List of service.instance names",
    )

    return parser


def main() -> None:
    args = get_parser().parse_args()

    if args.smartstack_readiness_check_enabled:
        smartstack_ready = is_smartstack_ready(
            synapse_host=synapse_host,
            synapse_port=synapse_port,
            synapse_haproxy_url_format=synapse_haproxy_url_format,
            services=args.services,
            host_ip=pod_ip,
            host_port=args.pod_port,
        )
    else:
        smartstack_ready = True

    if args.envoy_readiness_check_enabled:
        if args.envoy_check_mode == "admin-port":
            envoy_ready = is_envoy_ready(
                envoy_host=envoy_host,
                envoy_admin_port=envoy_admin_port,
                envoy_admin_endpoint_format=envoy_admin_endpoint_format,
                registrations=args.services,
                pod_ip=pod_ip,
                pod_port=args.pod_port,
            )
        elif args.envoy_check_mode == "eds-dir":
            envoy_ready = are_namespaces_up_in_eds(
                envoy_eds_path=envoy_eds_path,
                namespaces=args.services,
                pod_ip=pod_ip,
                pod_port=args.pod_port,
            )

    else:
        envoy_ready = True

    if smartstack_ready and envoy_ready:
        sys.exit(0)
    else:
        if not smartstack_ready:
            print(
                f"Could not find backend {pod_ip}:{args.pod_port} for service {args.services} "
                f"on Haproxy at {synapse_host}:{synapse_port}"
            )
        if not envoy_ready:
            print(
                f"Could not find backend {pod_ip}:{args.pod_port} for service {args.services} "
                f"on Envoy at {envoy_host}"
            )
        sys.exit(1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
import argparse
import asyncio
import logging
from typing import Type

import pysensu_yelp

from paasta_tools.eks_tools import EksDeploymentConfig
from paasta_tools.instance import kubernetes as pik
from paasta_tools.kubernetes_tools import get_kubernetes_app_name
from paasta_tools.kubernetes_tools import KubeClient
from paasta_tools.kubernetes_tools import KubernetesDeploymentConfig
from paasta_tools.metrics.metastatus_lib import suffixed_number_value
from paasta_tools.monitoring_tools import send_event
from paasta_tools.paasta_service_config_loader import PaastaServiceConfigLoader
from paasta_tools.utils import DEFAULT_SOA_DIR
from paasta_tools.utils import list_services
from paasta_tools.utils import load_system_paasta_config
from paasta_tools.utils import SystemPaastaConfig

log = logging.getLogger(__name__)


def parse_args():
    parser = argparse.ArgumentParser(
        description=(
            "Check all autoscaled services to see if they're at their max_instances."
            " If so, send an alert if their utilization is above max_instances_alert_threshold."
        )
    )
    parser.add_argument(
        "-d",
        "--soa-dir",
        dest="soa_dir",
        default=DEFAULT_SOA_DIR,
        help="Use a different soa config directory",
    )
    parser.add_argument(
        "--dry-run",
        dest="dry_run",
        action="store_true",
        help="Print Sensu alert events instead of sending them",
    )
    return parser.parse_args()


async def check_max_instances(
    soa_dir: str,
    cluster: str,
    instance_type_class: Type[KubernetesDeploymentConfig],
    system_paasta_config: SystemPaastaConfig,
    dry_run: bool = False,
):
    kube_client = KubeClient()
    for service in list_services(soa_dir=soa_dir):
        service_config = PaastaServiceConfigLoader(service=service, soa_dir=soa_dir)
        for job_config in service_config.instance_configs(
            cluster=cluster, instance_type_class=instance_type_class
        ):
            instance = job_config.get_instance()
            if not job_config.get_autoscaling_metric_spec(
                name=get_kubernetes_app_name(service, instance),
                cluster=cluster,
                kube_client=kube_client,
                namespace=job_config.get_namespace(),
            ):
                # Not an instance that uses HPA, don't check.
                # TODO: should we send status=0 here, in case someone disables autoscaling for their service / changes
                # to bespoke autoscaler?
                continue

            if not job_config.get_docker_image():
                # skip services that haven't been marked for deployment yet.
                continue

            autoscaling_status = await pik.autoscaling_status(
                kube_client=kube_client,
                job_config=job_config,
                namespace=job_config.get_namespace(),
            )
            if autoscaling_status["min_instances"] == -1:
                log.warning(
                    f"HPA {job_config.get_sanitised_deployment_name()} not found."
                )
                continue

            if (
                autoscaling_status["min_instances"]
                == autoscaling_status["max_instances"]
            ) and "canary" in instance:
                status = pysensu_yelp.Status.OK
                output = f'Not checking {service}.{instance} as the instance name contains "canary" and min_instances == max_instances.'
            elif (
                autoscaling_status["desired_replicas"]
                >= autoscaling_status["max_instances"]
            ):
                threshold = job_config.get_autoscaling_max_instances_alert_threshold()
                setpoint = job_config.get_autoscaling_params()["setpoint"]
                metric_threshold_target_ratio = threshold / setpoint

                status = pysensu_yelp.Status.UNKNOWN
                output = "how are there no metrics for this thing?"
                for metric in autoscaling_status["metrics"]:
                    current_value = suffixed_number_value(metric["current_value"])
                    target_value = suffixed_number_value(metric["target_value"])

                    if current_value / target_value > metric_threshold_target_ratio:
                        status = pysensu_yelp.Status.CRITICAL
                        output = f"{service}.{instance}: Ratio of current value to target value ({current_value} / {target_value}) is greater than the ratio of max_instances_alert_threshold to setpoint ({threshold} / {setpoint})"
                    else:
                        status = pysensu_yelp.Status.OK
                        output = f"{service}.{instance}: Ratio of current value to target value ({current_value} / {target_value}) is below the ratio of max_instances_alert_threshold to setpoint ({threshold} / {setpoint})"
            else:
                status = pysensu_yelp.Status.OK
                output = f"{service}.{instance} is below max_instances."

            monitoring_overrides = job_config.get_monitoring()
            monitoring_overrides.update(
                {
                    "page": False,  # TODO: remove this line once this alert has been deployed for a little while.
                    "runbook": "y/check-autoscaler-max-instances",
                    "tip": (
                        "The autoscaler wants to scale up to handle additional load because your service is overloaded, but cannot scale any higher because of max_instances. You may want to bump max_instances. To make this alert quieter, adjust autoscaling.max_instances_alert_threshold in yelpsoa-configs."
                    ),
                }
            )
            send_event(
                service,
                check_name=f"check_autoscaler_max_instances.{service}.{instance}",
                overrides=monitoring_overrides,
                status=status,
                output=output,
                soa_dir=soa_dir,
                ttl=None,
                cluster=cluster,
                system_paasta_config=system_paasta_config,
                dry_run=dry_run,
            )


def main():
    args = parse_args()
    system_paasta_config = load_system_paasta_config()

    for instance_type_class in [KubernetesDeploymentConfig, EksDeploymentConfig]:
        asyncio.run(
            check_max_instances(
                soa_dir=args.soa_dir,
                cluster=system_paasta_config.get_cluster(),
                instance_type_class=instance_type_class,
                system_paasta_config=system_paasta_config,
                dry_run=args.dry_run,
            )
        )


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# Copyright 2015-2016 Yelp Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from paasta_tools.autoscaling import load_boost
from paasta_tools.cli.utils import execute_paasta_cluster_boost_on_remote_master
from paasta_tools.cli.utils import lazy_choices_completer
from paasta_tools.utils import DEFAULT_SOA_DIR
from paasta_tools.utils import list_clusters
from paasta_tools.utils import load_system_paasta_config


def add_subparser(subparsers):
    boost_parser = subparsers.add_parser(
        "boost",
        help="Set, print the status, or clear a capacity boost for a given region in a PaaSTA cluster",
        description=(
            "'paasta boost' is used to temporarily provision more capacity in a given cluster "
            "It operates by ssh'ing to a Mesos master of a remote cluster, and "
            "interacting with the boost in the local zookeeper cluster. If you set or clear "
            "a boost, you may want to run the cluster autoscaler manually afterward."
        ),
        epilog=(
            "The boost command may time out during heavy load. When that happens "
            "users may execute the ssh command directly, in order to bypass the timeout."
        ),
    )
    boost_parser.add_argument(
        "-v",
        "--verbose",
        action="count",
        dest="verbose",
        default=0,
        help="""Print out more output regarding the state of the cluster.
        Multiple v options increase verbosity. Maximum is 3.""",
    )
    boost_parser.add_argument(
        "-c",
        "--cluster",
        type=str,
        required=True,
        help="""Paasta cluster(s) to boost. This option can take comma separated values.
        If auto-completion doesn't work, you can get a list of cluster with `paasta list-clusters'""",
    ).completer = lazy_choices_completer(list_clusters)
    boost_parser.add_argument(
        "--soa-dir",
        dest="soa_dir",
        metavar="SOA_DIR",
        default=DEFAULT_SOA_DIR,
        help="define a different soa config directory",
    )
    boost_parser.add_argument(
        "-p",
        "--pool",
        type=str,
        default="default",
        help="Name of the pool you want to increase the capacity. Default is 'default' pool.",
    )
    boost_parser.add_argument(
        "-b",
        "--boost",
        type=float,
        default=load_boost.DEFAULT_BOOST_FACTOR,
        help="Boost factor to apply. Default is 1.5. A big failover should be 2, 3 is the max.",
    )
    boost_parser.add_argument(
        "-d",
        "--duration",
        type=int,
        default=load_boost.DEFAULT_BOOST_DURATION,
        help="Duration of the capacity boost in minutes. Default is 40",
    )
    boost_parser.add_argument(
        "-f",
        "--force",
        action="store_true",
        dest="override",
        help="Replace an existing boost. Default is false",
    )
    boost_parser.add_argument(
        "action",
        choices=["set", "status", "clear"],
        help="You can view the status, set or clear a boost.",
    )
    boost_parser.set_defaults(command=paasta_boost)


def paasta_boost(args):
    soa_dir = args.soa_dir
    system_paasta_config = load_system_paasta_config()
    all_clusters = list_clusters(soa_dir=soa_dir)
    clusters = args.cluster.split(",")
    for cluster in clusters:
        if cluster not in all_clusters:
            print(
                f"Error: {cluster} doesn't look like a valid cluster. "
                + "Here is a list of valid paasta clusters:\n"
                + "\n".join(all_clusters)
            )
            return 1

    return_code, output = execute_paasta_cluster_boost_on_remote_master(
        clusters=clusters,
        system_paasta_config=system_paasta_config,
        action=args.action,
        pool=args.pool,
        duration=args.duration if args.action == "set" else None,
        override=args.override if args.action == "set" else None,
        boost=args.boost if args.action == "set" else None,
        verbose=args.verbose,
    )
    print(output)
    return return_code
